--------------------------------------------------------------------------------------
-- PetFarmChallengeHelper - 
--------------------------------------------------------------------------------------
-- PetFarmChallengeHelper.lua 
--
--[[

PetFarmChallengeHelper is a temporary solution to allow Pet Farm Challenges using
LA/LAC. 

It detects dropped BOP pets and converts the item IDs to those of caged pets 
which have a real item value.

Author: Caraxe
All Rights Reserved

Version 0.1.3 - 2018.08.27

------------------------------------------------------------------------------------]]

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceEvent-3.0", "AceConsole-3.0")

addon.METADATA = {
	NAME = GetAddOnMetadata(..., "Title"), 
	VERSION = GetAddOnMetadata(..., "Version")
}

local TSMAPI_Item_GetLink = nil
local TSMAPI_MoneyToString = nil
local TSMAPI_Item_ToItemID = nil

local LA = LibStub("AceAddon-3.0"):GetAddon("LootAppraiser", true)
local LAC = LibStub("AceAddon-3.0"):GetAddon("LootAppraiser Challenge", true)

local OldLATSMGIV = nil
local function NewLATSMGIV(tab, itemID, priceSource)
	local newItemString = PfchItemID2Species(itemID)

	-- print("|cFF33FF99" .. addonName .. "|r:", "itemID " .. tostring(itemID) .. " itemString " .. newItemString);

	local oldliv = OldLATSMGIV(tab, itemID, priceSource) or 0
	local newliv = OldLATSMGIV(tab, newItemString, priceSource) or 0
	if not (oldliv == newliv) then
		print("|cFF33FF99" .. addonName .. "|r:", "changed pet value " .. 
			TSMAPI_Item_GetLink(itemID) .. 
			" from " .. TSMAPI_MoneyToString(oldliv, "OPT_PAD") .. 
			" to " .. TSMAPI_MoneyToString(newliv, "OPT_PAD"));
	end

	return OldLATSMGIV(tab, newItemString, priceSource)
end	

local OldLACIS = nil
local function NewLACIS(itemLink)
	local itemID = TSMAPI_Item_ToItemID(itemLink)
	local newItemString = PfchItemID2Species(itemID)

	-- print("|cFF33FF99" .. addonName .. "|r:", "itemLink " .. itemLink .. " itemID " .. tostring(itemID) .. " itemString " .. newItemString);
	
	if (itemID == newItemString) then
		return OldLACIS(itemLink)
	else
		print("|cFF33FF99" .. addonName .. "|r:", itemLink .. " detected in LAC");
		return OldLACIS(newItemString)
	end
end	

function addon:OnEnable()
	-- TSM3
 	if TSMAPI and TSMAPI.Item and TSMAPI.Item.GetLink and TSMAPI.MoneyToString and TSMAPI.Item.ToItemID then
		TSMAPI_Item_GetLink = function(...) return TSMAPI.Item:GetLink(...) end
		TSMAPI_MoneyToString = function(...) return TSMAPI:MoneyToString(...) end
		TSMAPI_Item_ToItemID = function(...) return TSMAPI.Item:ToItemID(...) end
	end
	-- TSM4
	if TSMAPI_FOUR and TSMAPI_FOUR.Item and TSMAPI_FOUR.Item.GetLink and TSMAPI_FOUR.Item.GetLink then
		TSMAPI_Item_GetLink = TSMAPI_FOUR.Item.GetLink
		TSMAPI_Item_ToItemID = TSMAPI_FOUR.Item.GetLink
	end
	if TSM_API and TSM_API.FormatMoneyString then
		TSMAPI_MoneyToString = TSM_API.FormatMoneyString
		print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", " new FormatMoneyString API found.");
	elseif TSMAPI_FOUR and TSMAPI_FOUR.Money and TSMAPI_FOUR.Money.ToString then
		TSMAPI_MoneyToString = TSMAPI_FOUR.Money.ToString
	end

	if TSMAPI_Item_GetLink and TSMAPI_MoneyToString and TSMAPI_Item_ToItemID then
		if LA then
			-- caged pets are tradeable
			if (LA.TSM.GetItemValue) then
				OldLATSMGIV = LA.TSM.GetItemValue
				LA.TSM.GetItemValue = NewLATSMGIV
				print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", " hooked into LA " .. LA.METADATA.VERSION .. ", dropped pets have real values now.");
			end
			if LAC then
				-- caged pets are not soulbound
				if (LAC.IsSoulbound) then
					OldLACIS = LAC.IsSoulbound
					LAC.IsSoulbound = NewLACIS
					print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", " hooked into LAC " .. LAC.METADATA.VERSION .. ", dropped pets show up in LACs.");
				end
			else
				print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", " NOT hooked into LAC " .. LAC.METADATA.VERSION);
			end
		else
			print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", " NOT hooked into LA " .. LA.METADATA.VERSION);
		end
	else
		print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", "No TSM API found.");
	end
end

-- EOF
